@@ -9,11 +9,11 @@ function (formula, data = NULL, ..., subset, na.action = na.omit, scale = TRUE)
     if (!inherits(formula, "formula"))
         stop("method is only for formula objects")
     m <- match.call(expand.dots = FALSE)
-    if (identical(class(eval.parent(m$data)), "matrix"))
+    if (inherits(eval.parent(m$data), "matrix"))
         m$data <- as.data.frame(eval.parent(m$data))
     m$... <- NULL
     m$scale <- NULL
-    m[[1]] <- as.name("model.frame")
+    m[[1L]] <- quote(stats::model.frame)
     m$na.action <- na.action
     m <- eval(m, parent.frame())
     Terms <- attr(m, "terms")
@@ -213,6 +213,8 @@ function (x,
         }
 
     if (type < 3 && !is.null(class.weights)) {
+        if (is.character(class.weights) && class.weights == "inverse")
+            class.weights <- 1 / table(y)
         if (is.null(names(class.weights)))
             stop("Weights have to be specified along with their according level names !")
         weightlabels <- match (names(class.weights), lev)
@@ -246,7 +248,7 @@ function (x,
     if (is.null(sparse)) stop("sparse argument must not be NULL!")
     if (is.null(probability)) stop("probability argument must not be NULL!")
 
-    cret <- .C ("svmtrain",
+    cret <- .C (R_svmtrain,
                 ## data
                 as.double  (if (sparse) x@ra else t(x)),
                 as.integer (nr), as.integer(ncol(x)),
@@ -289,9 +291,9 @@ function (x,
                 cresults = double   (cross),
                 ctotal1  = double   (1),
                 ctotal2  = double   (1),
-                error    = err,
+                error    = err
 
-                PACKAGE = "e1071")
+                )
 
     if (cret$error != empty_string)
         stop(paste(cret$error, "!", sep=""))
@@ -318,8 +320,8 @@ function (x,
                  tot.nSV  = cret$nr, #total number of sv
                  nSV      = cret$nSV[1:cret$nclasses], #number of SV in diff. classes
                  labels   = cret$labels[1:cret$nclasses], #labels of the SVs.
-                 SV       = if (sparse) SparseM::t(SparseM::t(x[cret$index,]))
-                 else t(t(x[cret$index,])), #copy of SV
+                 SV       = if (sparse) SparseM::t(SparseM::t(x[cret$index]))
+                 else t(t(x[cret$index,,drop = FALSE])), #copy of SV
                  index    = cret$index,  #indexes of sv in x
                  ##constants in decision functions
                  rho      = cret$rho[1:(cret$nclasses * (cret$nclasses - 1) / 2)],
@@ -410,10 +412,10 @@ function (object, newdata,
         if (inherits(object, "svm.formula")) {
             if(is.null(colnames(newdata)))
                 colnames(newdata) <- colnames(object$SV)
-            newdata <- model.matrix(delete.response(terms(object)),
-                                    as.data.frame(newdata))
             newdata <- na.action(newdata)
             act <- attr(newdata, "na.action")
+            newdata <- model.matrix(delete.response(terms(object)),
+                                    as.data.frame(newdata))
         } else {
             newdata <- na.action(as.matrix(newdata))
             act <- attr(newdata, "na.action")
@@ -433,7 +435,7 @@ function (object, newdata,
     if (ncol(object$SV) != ncol(newdata))
         stop ("test data does not match model !")
 
-    ret <- .C ("svmpredict",
+    ret <- .C (R_svmpredict,
                as.integer (decision.values),
                as.integer (probability),
 
@@ -470,9 +472,9 @@ function (object, newdata,
                ## decision-values
                ret = double(nrow(newdata)),
                dec = double(nrow(newdata) * object$nclasses * (object$nclasses - 1) / 2),
-               prob = double(nrow(newdata) * object$nclasses),
+               prob = double(nrow(newdata) * object$nclasses)
+
 
-               PACKAGE = "e1071"
                )
 
     ret2 <- if (is.character(object$levels)) # classification: return factors
@@ -536,15 +538,16 @@ function (x, ...)
         cat("       cost: ", x$cost, "\n")
     if (x$kernel==1)
         cat("     degree: ", x$degree, "\n")
-    cat("      gamma: ", x$gamma, "\n")
+    if (x$type==1 || x$type==2 || x$type==3)
+        cat("      gamma: ", x$gamma, "\n")
     if (x$kernel==1 || x$kernel==3)
         cat("     coef.0: ", x$coef0, "\n")
     if (x$type==1 || x$type==2 || x$type==4)
         cat("         nu: ", x$nu, "\n")
     if (x$type==3) {
         cat("    epsilon: ", x$epsilon, "\n\n")
-        if (x$compprob)
-            cat("Sigma: ", x$sigma, "\n\n")
+    if (x$compprob)
+        cat("Sigma: ", x$sigma, "\n\n")
     }
 
     cat("\nNumber of Support Vectors: ", x$tot.nSV)
@@ -673,11 +676,11 @@ function(x, data, formula = NULL, fill = TRUE,
 }
 
 write.svm <-
-function (object, svm.file="Rdata.svm", scale.file = "Rdata.scale",
+function (object, svm.file = "Rdata.svm", scale.file = "Rdata.scale",
           yscale.file = "Rdata.yscale")
 {
 
-    ret <- .C ("svmwrite",
+    ret <- .C (R_svmwrite,
                ## model
                as.double  (if (object$sparse) object$SV@ra else t(object$SV)),
                as.integer (nrow(object$SV)), as.integer(ncol(object$SV)),
@@ -702,9 +705,9 @@ function (object, svm.file="Rdata.svm", scale.file = "Rdata.scale",
                as.double  (object$coef0),
 
                ## filename
-               as.character(svm.file),
+               as.character(svm.file)
+
 
-               PACKAGE = "e1071"
                )$ret
 
     write.table(data.frame(center = object$x.scale$"scaled:center",
@@ -716,3 +719,14 @@ function (object, svm.file="Rdata.svm", scale.file = "Rdata.scale",
                                scale  = object$y.scale$"scaled:scale"),
                     file=yscale.file, col.names=FALSE, row.names=FALSE)
 }
+
+coef.svm <- function(object, ...)
+{
+    if (object$kernel != 0 || object$nclasses > 2)
+        stop("Only implemented for regression or binary classification with linear kernel.")
+    ret <- drop(crossprod(object$coefs, object$SV))
+    trm <- object$terms
+    if(!is.null(trm))
+        names(ret) <- labels(trm)
+    c(`(Intercept)` = -object$rho, ret)
+}
